import { ILifetime } from '../Lifetime';
import { IServiceProvider } from '../ServiceProvider';
import { DependencyOptions, Factory, IServiceCollection, Key, LifetimeConstructor, Selector } from './IServiceCollection';
export declare type MockSetup<E> = {
    [key in keyof E]?: Partial<Required<E>[key]> | Factory<E[key], E>;
};
export declare type RecordCollection<E> = Record<Key<E>, ILifetime<unknown, E>>;
export declare class ServiceCollection<E = any> implements IServiceCollection<E> {
    private readonly lifetimes;
    replaceSingleton<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    replaceTransient<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    replaceScoped<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    replace<T>(Lifetime: LifetimeConstructor<T, E>, dependency: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    tryAddSingleton<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    tryAddTransient<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    tryAddScoped<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    tryAdd<T>(Lifetime: LifetimeConstructor<T, E>, dependency: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    addSingleton<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    addTransient<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    addScoped<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    add<T>(Lifetime: LifetimeConstructor<T, E>, dependency: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    get<T>(selector: Selector<T, E>): ILifetime<T, E> | undefined;
    remove<T>(selector: Selector<T, E>): ILifetime<T, E> | undefined;
    private extractDependency;
    static extractSelector<T, E>(options: Selector<T, E>): Key<E>;
    build(): IServiceProvider<E>;
    buildMock(mock?: MockSetup<E>): IServiceProvider<E>;
    private cloneLifetimes;
}
