"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceCollection = void 0;
var Lifetime_1 = require("../Lifetime");
var Errors_1 = require("../Errors");
var ServiceProvider_1 = require("../ServiceProvider");
var ServiceCollection = /** @class */ (function () {
    function ServiceCollection() {
        Object.defineProperty(this, "lifetimes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
    }
    Object.defineProperty(ServiceCollection.prototype, "replaceSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.replace(Lifetime_1.Singleton, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replaceTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.replace(Lifetime_1.Transient, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replaceScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.replace(Lifetime_1.Scoped, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replace", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, dependency, selector) {
            this.remove(selector);
            this.add(Lifetime, dependency, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAddSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.tryAdd(Lifetime_1.Singleton, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAddTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.tryAdd(Lifetime_1.Transient, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAddScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.tryAdd(Lifetime_1.Scoped, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAdd", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, dependency, selector) {
            try {
                this.add(Lifetime, dependency, selector);
            }
            catch (e) {
                var type = e.type;
                if (type !== Errors_1.DependencyErrorType.Duplicate)
                    throw e;
            }
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.add(Lifetime_1.Singleton, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.add(Lifetime_1.Transient, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.add(Lifetime_1.Scoped, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "add", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, dependency, selector) {
            var name = ServiceCollection.extractSelector(selector);
            if (name in this.lifetimes)
                throw new Errors_1.DuplicateDependencyError(name);
            var factory = this.extractDependency(dependency);
            this.lifetimes[name] = new Lifetime(name, factory);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "get", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var key = ServiceCollection.extractSelector(selector);
            return this.lifetimes[key];
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "remove", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var result = this.get(selector);
            if (result)
                delete this.lifetimes[result.name];
            return result;
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "extractDependency", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Option) {
            if (typeof Option === 'function')
                return function (p) { return new Option(p); };
            return Option.factory;
        }
    });
    Object.defineProperty(ServiceCollection, "extractSelector", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            switch (typeof options) {
                case 'function':
                    var proxy = new Proxy({}, { get: function (t, p) { return p.toString(); } });
                    return options(proxy);
                case 'symbol':
                    return options.toString();
                case 'string':
                    return options;
                default:
                    throw new Error("Name selector could not match anything");
            }
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "build", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            var lifetimes = this.cloneLifetimes();
            return new ServiceProvider_1.ServiceProvider(lifetimes);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "buildMock", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (mock) {
            var provider = this.build();
            Object.entries(provider.lifetimes)
                .map(function (_a) {
                var key = _a[0], value = _a[1];
                return ({
                    name: key,
                    lifetime: value,
                });
            })
                .forEach(function (_a) {
                var name = _a.name, lifetime = _a.lifetime;
                var mockSetup = mock && mock[name];
                var valueProxy = new Proxy({}, {
                    construct: function (target, prop) {
                        throw new Errors_1.ShouldBeMockedDependencyError(name.toString(), prop.toString(), 'construct');
                    },
                    apply: function (target, prop) {
                        throw new Errors_1.ShouldBeMockedDependencyError(name.toString(), prop.toString(), 'apply');
                    },
                    get: function (target, prop) {
                        throw new Errors_1.ShouldBeMockedDependencyError(name.toString(), prop.toString(), 'get');
                    },
                    set: function (target, prop) {
                        throw new Errors_1.ShouldBeMockedDependencyError(name.toString(), prop.toString(), 'set');
                    },
                });
                provider.lifetimes[name] = new Proxy(lifetime, {
                    get: function (target, prop) {
                        if (prop !== 'provide')
                            return target[prop];
                        return function (context) {
                            if (context.dependencyTracker.depth === 1)
                                return target.provide(context);
                            switch (typeof mockSetup) {
                                case 'undefined':
                                    return valueProxy;
                                case 'function':
                                    return mockSetup(context.proxy);
                                default:
                                    return mockSetup;
                            }
                        };
                    },
                });
            });
            return provider;
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "cloneLifetimes", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return Object.entries(this.lifetimes)
                .map(function (_a) {
                var key = _a[0], value = _a[1];
                return ({
                    name: key,
                    lifetime: value.clone(),
                });
            })
                .reduce(function (a, _a) {
                var name = _a.name, lifetime = _a.lifetime;
                a[name] = lifetime;
                return a;
            }, {});
        }
    });
    return ServiceCollection;
}());
exports.ServiceCollection = ServiceCollection;
